unit UMetBase;

interface

uses
  Winapi.Windows, vcl.Graphics, math,
  sysutils, PngImage, jpeg, gifImg ;

type
  matImg = record
    nc,nr  : integer;
    dat    : array of array of array of single;
  end;

  function ajusta255( z : single) : byte;

  procedure BitMap2Mat ( BB : TBitMap; var MM : MatImg   );
  procedure Mat2BitMap ( MM : MatImg ; var BB : TBitMap  );
  procedure DefMatriz  ( var MM : MatImg; nc,  nr : integer );
  procedure copiaMat   ( M1 : MatImg; var M2 : MatImg);
  procedure salvaImagen( BB : TBitMap; nombre : string );

var
  _lamb  : byte;

implementation

// Copia matriz en matriz
procedure copiaMat  ( M1 : MatImg; var M2 : MatImg);
var
  xx, yy, cc : integer;
begin
  DefMatriz(M2, M1.nc, M1.nr);

  for yy := 0 to M1.nr-1 do
    for xx := 0 to M1.nc-1 do
      for cc := 0 to 2 do
        M2.dat[xx][yy][cc] := M1.dat[xx][yy][cc];
end;

// Define tamao de matriz
procedure DefMatriz ( var MM : MatImg; nc,  nr : integer );
begin
  MM.nc := nc;
  MM.nr := nr;
  SetLength(MM.dat,nc,nr,3);
end;

// ajusta al intervalo [0 .. 255]
function ajusta255( z : single) : byte;
var
  sal : byte;
begin
  if z<0
    then sal := 0
    else if z>255
           then sal := 255
           else sal := ceil(z);

  result := sal;
end;

// Transforma de BitMap A Matriz
procedure BitMap2Mat( BB : TBitMap; var MM : MatImg );
var
  xx,yy,pix : integer;

begin
  MM.nc := BB.Width;
  MM.nr := BB.Height;
  SetLength(MM.dat, MM.nc, MM.nr, 3);

  for yy := 0 to MM.nr-1 do
    for xx := 0 to MM.nc-1 do begin
      pix := BB.Canvas.Pixels[xx,yy];
      MM.dat[xx][yy][0] := GetRValue(pix);
      MM.dat[xx][yy][1] := GetGValue(pix);
      MM.dat[xx][yy][2] := GetBValue(pix);
    end;
end;

// Transforma de Matriz a BitMap
procedure Mat2BitMap( MM : MatImg; var BB : TBitMap );
var
  xx,yy   : integer;
  r,g,b   : byte;

begin
  BB.Width  := MM.nc;
  BB.Height := MM.nr;

  for yy := 0 to MM.nr-1 do
    for xx := 0 to MM.nc-1 do begin
      r := ajusta255(MM.dat[xx][yy][0]);
      g := ajusta255(MM.dat[xx][yy][1]);
      b := ajusta255(MM.dat[xx][yy][2]);

      BB.Canvas.Pixels[xx,yy] := RGB(r,g,b);
    end;
end;

// Salva Imagen
procedure salvaImagen( BB : TBitMap; nombre : string );
var
  ext  : string;
  fpng : TPngImage;
  fjpg : TjpegImage;
  fgif : TGifImage;
begin
  ext := lowerCase(ExtractFileExt(nombre));

  if (ext = '.bmp') then begin
    BB.SaveToFile(nombre);
  end;

  if (ext = '.png') then begin
    try
      fpng := TpngImage.Create;
      fpng.Assign(BB);
      fpng.SaveToFile(nombre);
    finally
      fpng.Free;
    end;
  end;

  if (ext = '.jpg') or (ext = '.jpeg') then begin
    try
      fjpg := TjpegImage.Create;
      fjpg.Assign(BB);
      fjpg.SaveToFile(nombre);
    finally
      fjpg.Free;
    end;
  end;

  if (ext = '.gif') then begin
    try
      fgif := TGifImage.Create;
      fgif.Assign(BB);
      fgif.SaveToFile(nombre);
    finally
      fgif.Free;
    end;
  end;

end;

begin
  _lamb := 255;

end.
